/*  pendu.c - this file is part of Pendu, a simple hangman game written in GTK3+
    Copyright 2011-2012 RIVIÈRE Aurélien <aurelien.riv@gmail.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301, USA.
*/

#include "pendu.h"

static Level lv;
static Stats sta;
static GtkWidget *cover, *odds, *hstry, *input, *canvas;

void new_game (GtkWidget *w, void *p)
{
	(void) w; (void) p;
	
    switch (lv.level)
    {
        case 1: sta.l1.total++; break;
        case 2: sta.l2.total++; break;
        case 3: sta.l3.total++; break;
        case 4: sta.l4.total++; break;
    }

    lv.remHits = lv.maxHits;
    lv.guessed = 0;

    newWord(&lv);
    gtk_label_set_markup (GTK_LABEL (cover), g_markup_printf_escaped ("<b>%s</b>", lv.mask));    
    gtk_label_set_text (GTK_LABEL (odds), g_strdup_printf ( _("Remaining tests : %i"), lv.remHits));
    gtk_image_set_from_file (GTK_IMAGE (canvas), g_strdup_printf(PICTURES "pendu-%i.png", lv.img[0]));
    gtk_label_set_text (GTK_LABEL (hstry), NULL);
    gtk_entry_set_text (GTK_ENTRY(input), "");
}

void confirm_entry (GtkWidget *w, void *p)
{
    char car = toupper( *gtk_entry_get_text (GTK_ENTRY(w)));
    (void) p;

    if(!lv.guessed && lv.remHits > 0 && car >= 'A' && car <= 'Z')
    {
        unsigned short addHst = 1, decrement = 1, i, imin = 0, imax = strlen(lv.wd);
        if (lv.sfl) imin = 1; imax--;
        
        for (i=imin; i <= imax; i++)
        {
            if      (car == lv.mask[i])  { addHst = 0; break; }                            /* The letter has already been found */
            else if (car == lv.wd[i])    { lv.mask[i] = car; addHst = 0, decrement = 0; }  /* The letter belong to the word */
            if (!strcmp(lv.mask, lv.wd)) { lv.guessed = 1; break; }                        /* Checking if the whole word had been guessed */
        }

        if(addHst)
        {
            const char *hst = gtk_label_get_text (GTK_LABEL(hstry));
            if (!strchr(hst, car)) gtk_label_set_text (GTK_LABEL (hstry), g_strdup_printf ("%s %c", hst, car ));
        }

        if (decrement) lv.remHits--;
        if (!lv.remHits)
        {
            char *str = (char *)"";
            for (i=0; i<strlen(lv.wd); i++)
            {
                if (lv.mask[i] == lv.wd[i]) str = g_strdup_printf ("%s<span weight='bold'>%c</span>", str, lv.wd[i]);
                else                        str = g_strdup_printf ("%s<span color='red' weight='bold'>%c</span>", str, lv.wd[i]);
            }
            gtk_label_set_markup (GTK_LABEL (cover), str);
            g_free(str);

            gtk_image_set_from_file (GTK_IMAGE (canvas), PICTURES "hanged.png");
        }
        else
        {
            gtk_image_set_from_file (GTK_IMAGE (canvas), g_strdup_printf ( PICTURES "pendu-%i.png", lv.img[ lv.maxHits-lv.remHits ] ));
            gtk_label_set_markup (GTK_LABEL (cover), g_markup_printf_escaped ("<b>%s</b>", lv.mask));
        }
        gtk_label_set_text (GTK_LABEL (odds), g_strdup_printf ( _("Remaining tests : %i"), lv.remHits));

        if(lv.guessed)
        {
            switch (lv.level)
            {
                case 1: sta.l1.vic++; break;
                case 2: sta.l2.vic++; break;
                case 3: sta.l3.vic++; break;
                case 4: sta.l4.vic++; break;
            }
            gtk_label_set_markup (GTK_LABEL (cover), g_strdup_printf ("<span color='green' weight='bold'>%s</span>", lv.wd));
            gtk_image_set_from_file (GTK_IMAGE (canvas), PICTURES "saved.png");
        }
        gtk_entry_set_text (GTK_ENTRY(w), "");
    }
}

void exec (GtkWidget *w, char *arg)
{
	(void) w;
	system(arg);
}

void about (GtkWidget *w, void *p)
{
	GtkWidget *dialog;
    const char *authors[] = {"Aurelien RIVIERE <aurelien.riv@gmail.com> (maintainer)", "Maxime VILLARD <cypherique@ymail.com> (OpenBSD maintainer)", NULL};

	(void) w; (void) p;
	
    dialog = gtk_about_dialog_new ();
    gtk_about_dialog_set_program_name (GTK_ABOUT_DIALOG (dialog), "Pendu");
    gtk_about_dialog_set_logo_icon_name (GTK_ABOUT_DIALOG (dialog), "pendu");
    gtk_about_dialog_set_comments (GTK_ABOUT_DIALOG (dialog), _("Simple hangman game in GTK3+"));
    gtk_about_dialog_set_version (GTK_ABOUT_DIALOG (dialog), VERSION);
    gtk_about_dialog_set_copyright (GTK_ABOUT_DIALOG (dialog), "Copyright \xc2\xa9 2011-2012 Aurélien RIVIÈRE");
    gtk_about_dialog_set_website (GTK_ABOUT_DIALOG (dialog), HOMEPAGE);
    gtk_about_dialog_set_website_label (GTK_ABOUT_DIALOG (dialog), _("Website"));
    
    gtk_about_dialog_set_authors (GTK_ABOUT_DIALOG (dialog), authors);
    gtk_about_dialog_set_translator_credits (GTK_ABOUT_DIALOG (dialog), _("translator-credits"));
    gtk_about_dialog_set_license_type (GTK_ABOUT_DIALOG (dialog), GTK_LICENSE_GPL_3_0);
    
    switch (gtk_dialog_run(GTK_DIALOG(dialog))) { 
		case GTK_RESPONSE_CANCEL:
		case GTK_RESPONSE_DELETE_EVENT: 
			gtk_widget_destroy(dialog); 
	};
}

void goodbye (GtkWidget *w, void *p)
{
	(void) w, (void) p;
    ch_stats_file(&sta);
    ch_cfg_file (lv.level, lv.lg);
    gtk_main_quit();
}

int main (int argc, char *argv[])
{
	FILE *cfg = NULL;

	GtkWidget *window, *mainBox, *appBox, *box;
	GtkWidget *file, *game, *help, *item, *lv1_item, *lv2_item, *lv3_item, *lv4_item;

	GtkAccelGroup *accel;
	GSList *group = NULL;

    srand(time(NULL));

    setlocale(LC_ALL, "");
    textdomain("pendu");
    bindtextdomain("pendu", TRANSLATIONS);

    gtk_init (&argc, &argv);

    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title (GTK_WINDOW (window), _("Pendu"));
    gtk_window_set_icon_from_file (GTK_WINDOW (window), PIXMAPS "pendu.png", NULL);
    gtk_window_set_resizable (GTK_WINDOW(window), FALSE);
    g_signal_connect (window, "destroy", G_CALLBACK (goodbye), NULL);

    mainBox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
    box = gtk_menu_bar_new ();
    accel = gtk_accel_group_new();

        file = gtk_menu_new ();
            item = gtk_menu_item_new_with_label ( _("Statistics"));
            g_object_set_data (G_OBJECT (item), "level", &lv.level);
            g_object_set_data (G_OBJECT (item), "guessed", &lv.guessed);
            gtk_widget_add_accelerator(item, "activate", accel, GDK_KEY_S, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
            g_signal_connect ( G_OBJECT(item), "activate", G_CALLBACK (stats), &sta);
            gtk_menu_shell_append (GTK_MENU_SHELL(file), item);

            item = gtk_menu_item_new_with_label ( _("Preferences"));
            g_object_set_data (G_OBJECT (item), "level", &lv.level);
            gtk_widget_add_accelerator(item, "activate", accel, GDK_KEY_P, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
            g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(create_settings_ui), &lv);
            gtk_menu_shell_append (GTK_MENU_SHELL(file), item);

            item = gtk_menu_item_new_with_label ( _("Quit"));
            gtk_widget_add_accelerator(item, "activate", accel, GDK_KEY_Q, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
            g_signal_connect ( G_OBJECT(item), "activate", G_CALLBACK (goodbye), NULL);
            gtk_menu_shell_append (GTK_MENU_SHELL(file), item);
        item = gtk_menu_item_new_with_label ( _("File"));
        gtk_menu_item_set_submenu (GTK_MENU_ITEM(item), file);
        gtk_menu_shell_append(GTK_MENU_SHELL(box), item);

        game = gtk_menu_new ();
            item = gtk_menu_item_new_with_mnemonic (_("New"));
            g_signal_connect ( G_OBJECT(item), "activate", G_CALLBACK (new_game), NULL);
            gtk_widget_add_accelerator(item, "activate", accel, GDK_KEY_N, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
            gtk_menu_shell_append (GTK_MENU_SHELL(game), item);

            gtk_menu_shell_append (GTK_MENU_SHELL(game), gtk_separator_menu_item_new ());
            
            lv1_item = gtk_radio_menu_item_new_with_label (group, _("Easy"));
            group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (lv1_item));
            g_object_set_data(G_OBJECT(lv1_item), "LEVEL", GINT_TO_POINTER(1));
            g_signal_connect ( G_OBJECT(lv1_item), "toggled", G_CALLBACK (chLevel), &lv);
            gtk_menu_shell_append (GTK_MENU_SHELL(game), lv1_item);

            lv2_item = gtk_radio_menu_item_new_with_label (group, _("Medium"));
            group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (lv2_item));
            g_object_set_data(G_OBJECT(lv2_item), "LEVEL", GINT_TO_POINTER(2));
            g_signal_connect ( G_OBJECT(lv2_item), "toggled", G_CALLBACK (chLevel), &lv);
            gtk_menu_shell_append (GTK_MENU_SHELL(game), lv2_item);

            lv3_item = gtk_radio_menu_item_new_with_label (group, _("Hard"));
            group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (lv3_item));
            g_object_set_data(G_OBJECT(lv3_item), "LEVEL", GINT_TO_POINTER(3));
            g_signal_connect ( G_OBJECT(lv3_item), "toggled", G_CALLBACK (chLevel), &lv);
            gtk_menu_shell_append (GTK_MENU_SHELL(game), lv3_item);

            lv4_item = gtk_radio_menu_item_new_with_label (group, _("Sudden Death"));
            group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (lv4_item));
            g_object_set_data ( G_OBJECT(lv4_item), "LEVEL", GINT_TO_POINTER(4));
            g_signal_connect ( G_OBJECT(lv4_item), "toggled", G_CALLBACK (chLevel), &lv);
            gtk_menu_shell_append (GTK_MENU_SHELL(game), lv4_item);
        lv1_item = gtk_menu_item_new_with_label ( _("Game"));
        gtk_menu_item_set_submenu (GTK_MENU_ITEM(lv1_item), game);
        gtk_menu_shell_append(GTK_MENU_SHELL(box), lv1_item);

        help = gtk_menu_new ();
            item = gtk_menu_item_new_with_label ( _("Help"));
            gtk_widget_add_accelerator(item, "activate", accel, GDK_KEY_F1, FALSE, GTK_ACCEL_VISIBLE);
            g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(exec), g_strdup_printf("yelp man:pendu"));
            gtk_menu_shell_append (GTK_MENU_SHELL(help), item);

            item = gtk_menu_item_new_with_label ( _("Report a bug"));
            g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(exec), (char *)"xdg-open https://bugs.launchpad.net/pendu-gtk");
            gtk_menu_shell_append (GTK_MENU_SHELL(help), item);

            gtk_menu_shell_append (GTK_MENU_SHELL(help), gtk_separator_menu_item_new ());

            item = gtk_menu_item_new_with_label ( _("About"));
            g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(about), NULL);
            gtk_menu_shell_append (GTK_MENU_SHELL(help), item);
        item = gtk_menu_item_new_with_label ( _("Help"));
        gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), help);
        gtk_menu_shell_append (GTK_MENU_SHELL(box), item);

    gtk_window_add_accel_group(GTK_WINDOW(window), accel);
    gtk_box_pack_start(GTK_BOX(mainBox), box, FALSE, FALSE, 0);

    appBox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 5);
    box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);

        cover = gtk_label_new (NULL);
        gtk_box_pack_start (GTK_BOX (box), cover, TRUE, TRUE, 0);

        odds = gtk_label_new (NULL);
        gtk_box_pack_start (GTK_BOX (box), odds, FALSE, FALSE, 0);

		gtk_box_pack_start (GTK_BOX (box), gtk_label_new (_("Letters tested :")), FALSE, FALSE, 0);

		hstry = gtk_label_new (NULL);
		gtk_box_pack_start (GTK_BOX (box), hstry, FALSE, FALSE, 0);

        input = gtk_entry_new ();
        gtk_entry_set_max_length (GTK_ENTRY(input), 1);
        g_signal_connect (input, "activate", G_CALLBACK (confirm_entry), NULL);
        gtk_box_pack_start (GTK_BOX (box), input, FALSE, FALSE, 0);
        gtk_widget_set_margin_left(input, 45);
        gtk_widget_set_margin_right(input, 45);

    gtk_container_add (GTK_CONTAINER (appBox), box);

    box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);

        canvas = gtk_image_new ();
        gtk_widget_set_margin_right(canvas, 5);
        gtk_box_pack_start (GTK_BOX (box), canvas, FALSE, FALSE, 5);

        item = gtk_button_new_with_label(_("New"));
        gtk_widget_set_margin_right(item, 5);
        gtk_button_set_focus_on_click (GTK_BUTTON(item), FALSE);
        g_signal_connect (item, "clicked", G_CALLBACK (new_game), NULL);
        gtk_box_pack_start (GTK_BOX (box), item, FALSE, FALSE, 0);

    gtk_container_add (GTK_CONTAINER (appBox), box);
    gtk_container_add (GTK_CONTAINER (mainBox), appBox);
    gtk_container_add (GTK_CONTAINER (window), mainBox);

    loadStats(&sta);

    cfg = fopen ( CFG_PATH CFG_FILE, "r" );
    
    if (cfg && fscanf(cfg, "level = %hu\nlanguage = %s", &lv.level, lv.lg)) fclose(cfg);
    else
    {
        lv.level = 1;
        
        if (strcmp(_("_DICT_"), "_DICT_")) strcpy(lv.lg, _("_DICT_"));
		else strcpy(lv.lg, "en");

        ch_cfg_file (1, lv.lg);
    }

    switch (lv.level)
    {
        case 1: init_lv(&lv, 1, 0); break;
        case 2: gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM(lv2_item), TRUE); break;
        case 3: gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM(lv3_item), TRUE); break;
        case 4: gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM(lv4_item), TRUE); break;
    }

    gtk_widget_show_all (window);
    gtk_main ();

    return EXIT_SUCCESS;
}
