/*  settings.c - this file is part of Pendu, a simple hangman game written in GTK3+
    Copyright 2012 RIVIÈRE Aurélien <aurelien.riv@gmail.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301, USA.
*/

#include "settings.h"

#define DO_BOX_V(label) box_v = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);   gtk_notebook_append_page (GTK_NOTEBOOK(lv_notebook), box_v, gtk_label_new(_(label)));
#define DO_BOX_H        box_h = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0); gtk_box_pack_start(GTK_BOX(box_v), box_h, TRUE, TRUE, 0);

#define NUMHITS DO_BOX_H gtk_box_pack_start(GTK_BOX(box_h), gtk_label_new(_("Number of hits :")), TRUE, TRUE, 0);
#define MINSIZE DO_BOX_H gtk_box_pack_start(GTK_BOX(box_h), gtk_label_new(_("Word min size :")), TRUE, TRUE, 0);
#define MAXSIZE DO_BOX_H gtk_box_pack_start(GTK_BOX(box_h), gtk_label_new(_("Word max size :")), TRUE, TRUE, 0);

#define DO_SPI(name, val, min, max)																	     \
    name = gtk_spin_button_new_with_range (min, max, 1);							 					 \
    gtk_spin_button_set_value (GTK_SPIN_BUTTON(name), val);												 \
    g_signal_connect (name, "value-changed", G_CALLBACK (spin_value_changed), &val); 					 \
    gtk_box_pack_start(GTK_BOX(box_h), name, TRUE, TRUE, 0);

#define DO_SWI(name, val) 																			     \
    DO_BOX_H																							 \
    gtk_box_pack_start(GTK_BOX(box_h), gtk_label_new(_("Show first and last letter :")), TRUE, TRUE, 5); \
    name = gtk_switch_new ();																			 \
    gtk_switch_set_active (GTK_SWITCH(name), val);														 \
    g_signal_connect (name, "notify::active", G_CALLBACK (switch_value_changed), &val);					 \
    gtk_box_pack_start(GTK_BOX(box_h), name, TRUE, TRUE, 0);

#define DO_RADIO(name, label, cmp)																    	 \
    name = gtk_radio_button_new_with_label (group, _(label));											 \
    group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (name));										 \
    g_object_set_data(G_OBJECT(name), "LG", (char *) cmp);												 \
    g_signal_connect ( G_OBJECT(name), "toggled", G_CALLBACK (radio_value_changed), language);		     \
    gtk_box_pack_start (GTK_BOX (box_v), name, FALSE, FALSE, 0);									     \
    if ( !strcmp(lv->lg , cmp) )  gtk_toggle_button_set_active ( GTK_TOGGLE_BUTTON(name), TRUE);
    
void create_settings_ui (GtkWidget *w, Level *lv)
{
	char language[6];
	short *level;

	GtkWidget *dialog, *body, *notebook, *lv_notebook, *box_v, *box_h, *ctl;
	GSList *group = NULL;

    Level lv1, lv2, lv3, lv4;
    init_lv (&lv1, 1, 1);
    init_lv (&lv2, 2, 1);
    init_lv (&lv3, 3, 1);
    init_lv (&lv4, 4, 1);

    strcpy(language, lv->lg);

    dialog = gtk_dialog_new_with_buttons(_("Preferences"), NULL, GTK_DIALOG_MODAL, GTK_STOCK_DISCARD,GTK_RESPONSE_DELETE_EVENT, GTK_STOCK_APPLY,GTK_RESPONSE_APPLY, NULL);
    body = gtk_dialog_get_content_area (GTK_DIALOG (dialog));

    notebook = gtk_notebook_new();
	lv_notebook = gtk_notebook_new();
	gtk_notebook_set_tab_pos (GTK_NOTEBOOK(lv_notebook), GTK_POS_LEFT);
	    DO_BOX_V("Easy")
		NUMHITS DO_SPI(ctl, lv1.maxHits, 1, 13)
		MINSIZE DO_SPI(ctl, lv1.lmin, 5, 20)
		MAXSIZE DO_SPI(ctl, lv1.lmax, 5, 20)
				DO_SWI(ctl, lv1.sfl)
	    DO_BOX_V("Medium")
		NUMHITS DO_SPI(ctl, lv2.maxHits, 1, 13)
		MINSIZE DO_SPI(ctl, lv2.lmin, 5, 20)
		MAXSIZE DO_SPI(ctl, lv2.lmax, 5, 20)
				DO_SWI(ctl, lv2.sfl)
	    DO_BOX_V("Hard")
		NUMHITS DO_SPI(ctl, lv3.maxHits, 1, 13)
		MINSIZE DO_SPI(ctl, lv3.lmin, 5, 20)
		MAXSIZE DO_SPI(ctl, lv3.lmax, 5, 20)
				DO_SWI(ctl, lv3.sfl)
	    DO_BOX_V("Sudden Death")
		NUMHITS DO_SPI(ctl, lv4.maxHits, 1, 13)
		MINSIZE DO_SPI(ctl, lv4.lmin, 5, 20)
		MAXSIZE DO_SPI(ctl, lv4.lmax, 5, 20)
				DO_SWI(ctl, lv4.sfl)
    gtk_notebook_append_page (GTK_NOTEBOOK(notebook), lv_notebook, gtk_label_new(_("Levels")));

    box_v = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
		DO_RADIO(ctl, "English", "en")
		DO_RADIO(ctl, "French",  "fr")
		DO_RADIO(ctl, "Italian", "it")
		DO_RADIO(ctl, "Spanish", "es")
    gtk_notebook_append_page (GTK_NOTEBOOK(notebook), box_v, gtk_label_new(_("Dictionary")));

    gtk_box_pack_start(GTK_BOX(body), notebook, TRUE, TRUE, 0);
    gtk_widget_show_all (body);

	level = g_object_get_data(G_OBJECT(w), "level");
    gtk_notebook_set_current_page (GTK_NOTEBOOK(lv_notebook), *level -1);

    switch (gtk_dialog_run(GTK_DIALOG(dialog)))
    {
        case GTK_RESPONSE_APPLY:
			ch_cfg_file (lv->level, language);
			strcpy(lv->lg, language);
			save_levels_file (&lv1, &lv2, &lv3, &lv4);
			init_lv(lv, lv->level, 0);
			gtk_widget_destroy(dialog);
			break;
        case GTK_RESPONSE_DELETE_EVENT: gtk_widget_destroy(dialog); break;
    }
}

void spin_value_changed (GtkSpinButton *w, short *val)
{
	*val = gtk_spin_button_get_value_as_int (w);
}

void switch_value_changed (GtkSwitch *w, GParamSpec *pspec, short *val)
{
	(void) pspec; *val = gtk_switch_get_active (w);
}

void radio_value_changed (GtkWidget *w, char *lg)
{
    strcpy(lg, g_object_get_data(G_OBJECT(w), "LG"));
}

void save_levels_file (Level *lv1, Level *lv2, Level *lv3, Level *lv4)
{
    FILE *file = NULL;

    mkdir(CFG_PATH, 0744);
    file = fopen(CFG_PATH LEVELS_CFG, "w");
    fprintf(file, "LEVEL 1\n\tcoupMax = %i\n\ttmin = %i\n\ttmax = %i\n\tsfl = %i\n", lv1->maxHits, lv1->lmin, lv1->lmax, lv1->sfl);
    fprintf(file, "LEVEL 2\n\tcoupMax = %i\n\ttmin = %i\n\ttmax = %i\n\tsfl = %i\n", lv2->maxHits, lv2->lmin, lv2->lmax, lv2->sfl);
    fprintf(file, "LEVEL 3\n\tcoupMax = %i\n\ttmin = %i\n\ttmax = %i\n\tsfl = %i\n", lv3->maxHits, lv3->lmin, lv3->lmax, lv3->sfl);
    fprintf(file, "LEVEL 4\n\tcoupMax = %i\n\ttmin = %i\n\ttmax = %i\n\tsfl = %i\n", lv4->maxHits, lv4->lmin, lv4->lmax, lv4->sfl);
    fclose(file);
}

void ch_cfg_file (short nv, char *lg)
{
	FILE *file = NULL;

    mkdir(CFG_PATH, 0744);
    file = fopen(CFG_PATH CFG_FILE, "w");
    fprintf (file, "level = %i\nlanguage = %s", nv, lg);
    fclose(file);
}
