/*  stats.c - this file is part of Pendu, a simple hangman game written in GTK3+
    Copyright 2011-2012 RIVIÈRE Aurélien <aurelien.riv@gmail.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301, USA.
*/

#include "stats.h"

#define DO_STAT_TAB(sta_lv, label) 							\
    if (!sta_lv.total) prct = 0;							\
    else prct = (sta_lv.vic * 100) / sta_lv.total; 			\
    gtk_notebook_append_page (GTK_NOTEBOOK(notebook),		\
        gtk_label_new ( 									\
            g_strdup_printf("%s : %i\n%s : %i\n%s : %i%%",  \
                _("Win"), sta_lv.vic,						\
                _("Total"), sta_lv.total, 					\
                _("Percentage"), prct)), 					\
        gtk_label_new(_(label)));
        
void stats(GtkWidget *w, Stats *sta)
{
    short prct, *level;
    GtkWidget *dialog, *body, *notebook;
    
    dialog = gtk_dialog_new_with_buttons(_("Statistics"), NULL, GTK_DIALOG_MODAL, GTK_STOCK_DELETE,GTK_RESPONSE_APPLY, GTK_STOCK_CLOSE,GTK_RESPONSE_DELETE_EVENT, NULL);
    body = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
    
    notebook = gtk_notebook_new();

        DO_STAT_TAB(sta->l1, "Easy")
        DO_STAT_TAB(sta->l2, "Medium")
        DO_STAT_TAB(sta->l3, "Hard")
        DO_STAT_TAB(sta->l4, "Sudden Death")

    gtk_box_pack_start(GTK_BOX(body), notebook, TRUE, TRUE, 0);
    gtk_widget_show_all (body);

    level = g_object_get_data(G_OBJECT(w), "level");
    gtk_notebook_set_current_page (GTK_NOTEBOOK(notebook), *level -1);
    
    switch (gtk_dialog_run(GTK_DIALOG(dialog)))
    {
        case GTK_RESPONSE_APPLY:
            reset_stats(sta, *level, (int)g_object_get_data(G_OBJECT(w), "guessed"));
            gtk_widget_destroy(dialog);
            break;
        case GTK_RESPONSE_DELETE_EVENT:
            gtk_widget_destroy(dialog);
            break;
    }
}

void reset_stats (Stats *sta, short level, short guessed)
{
    sta->l1.vic = sta->l1.total = sta->l2.vic = sta->l2.total = sta->l3.vic = sta->l3.total = sta->l4.vic = sta->l4.total = 0;
    
    if(!guessed)
    {
        switch (level)
        {
            case 1: sta->l1.total = 1; break;
            case 2: sta->l2.total = 1; break;
            case 3: sta->l3.total = 1; break;
            case 4: sta->l4.total = 1; break;
        }
    }

    ch_stats_file(sta);
}

void loadStats(Stats *sta)
{
    FILE *file = NULL;
    file = fopen (CFG_PATH STATS_FILE, "r" );

    if (file)
    {
        short vic, total;

        fscanf(file, "LEVEL 1\n\tvic = %hd\n\ttotal = %hd\n", &vic, &total);
        sta->l1.vic = vic;
        sta->l1.total = total;

        fscanf(file, "LEVEL 2\n\tvic = %hd\n\ttotal = %hd\n", &vic, &total);
        sta->l2.vic = vic;
        sta->l2.total = total;

        fscanf(file, "LEVEL 3\n\tvic = %hd\n\ttotal = %hd\n", &vic, &total);
        sta->l3.vic = vic;
        sta->l3.total = total;

        fscanf(file, "LEVEL 4\n\tvic = %hd\n\ttotal = %hd\n", &vic, &total);
        sta->l4.vic = vic;
        sta->l4.total = total;

        fclose(file);
    }
    else
    {
        reset_stats (sta, 0, 1);
        ch_stats_file (sta);
    }
}

void ch_stats_file(Stats *sta)
{
    FILE *file = NULL;
    mkdir(CFG_PATH, 0744);
    file = fopen(CFG_PATH STATS_FILE, "w");

    fprintf (file, "LEVEL 1\n\tvic = %i\n\ttotal = %i\n", sta->l1.vic, sta->l1.total);
    fprintf (file, "LEVEL 2\n\tvic = %i\n\ttotal = %i\n", sta->l2.vic, sta->l2.total);
    fprintf (file, "LEVEL 3\n\tvic = %i\n\ttotal = %i\n", sta->l3.vic, sta->l3.total);
    fprintf (file, "LEVEL 4\n\tvic = %i\n\ttotal = %i\n", sta->l4.vic, sta->l4.total);

    fclose(file);
}
